.icons-on .contenttype-contentcategoryconfiguration,
.icons-on .contenttype-contentcategorygroup,
.icons-on .contenttype-contentcategory {
    background: transparent url("../contenttypes-sprite.png") no-repeat scroll 0 -644px;
}
.icons-on .contenttype-contentcategoryconfiguration,
.icons-on .contenttype-contentcategorygroup,
.icons-on .contenttype-contentcategory {
    display: inline-block;
    line-height: 2em;
    padding-left: 20px !important;
}

#content div.categorized-elements {
    display: inline-block;
}

#content div.categorized-elements h2 {
    margin-bottom: 0.3em;
}

/* display number of elements smaller */
div.categorized-elements a.deactivated > span {
    font-size: 90%;
}

#content div.categorized-elements a:link,
#content div.categorized-elements a:hover,
#content div.categorized-elements a:visited {
    color: #666;
    /* added !important to override default behavior of base.css that removes border-bottom in table.listing a */
    border: 1px solid #ccc !important;
    background: #EDEDED;
    padding-left: 0.2em;
    padding-right: 0.2em;
    padding-top: 0.1em;
    padding-bottom: 0.3em;
    border-radius: 5px;
    /* force height and display for Chrome */
    height: 16px;
    display: inherit;
    line-height: 1em;
}

p.warn_filesize,
span.warn_filesize {
    color: red;
    cursor: help;
    font-weight: bold;
}
p.soft_warn_filesize,
span.soft_warn_filesize {
    font-weight: bold;
}

img.categorized_elements_more_infos_icon {
    padding-right: 0.2em;
}

div.tooltipster-content ul.many_elements_1_columns {
    columns: 1;
    -webkit-columns: 1;
    -moz-columns: 1;
}
div.tooltipster-content ul.many_elements_2_columns {
    columns: 2;
    -webkit-columns: 2;
    -moz-columns: 2;
}
div.tooltipster-content ul.many_elements_3_columns {
    columns: 3;
    -webkit-columns: 3;
    -moz-columns: 3;
}
div.tooltipster-content ul.many_elements_4_columns {
    columns: 4;
    -webkit-columns: 4;
    -moz-columns: 4;
}
div.tooltipster-content ul.many_elements_5_columns {
    columns: 5;
    -webkit-columns: 5;
    -moz-columns: 5;
}
div.tooltipster-content ul.many_elements_6_columns {
    columns: 6;
    -webkit-columns: 6;
    -moz-columns: 6;
}
div.tooltipster-content ul.many_elements_7_columns {
    columns: 7;
    -webkit-columns: 7;
    -moz-columns: 7;
}
div.tooltipster-content ul.many_elements_8_columns {
    columns: 8;
    -webkit-columns: 8;
    -moz-columns: 8;
}

div.tooltipster-content {
    font-size: 0.9em;
}
div.tooltipster-categorized-elements li {
    display: block;
    padding-top: 0.2em;
    padding-bottom: 0.2em;
}

/* on hover highlight background to see "connect" element title and details icons */
div.tooltipster-categorized-elements li:hover {
    background-color: #f0f0f0;
    border-radius: 5px;
}
/* avoid categorized element title and touching details icons */
div.tooltipster-categorized-elements li a {
    padding-right: 1em;
}

#content table.iconified-listing {
    width: 100%;
}

/* avoid occcasionnally scroll bars */
div.tooltipster-categorized-elements {
    min-width: 350px;
    max-width: 750px;
    padding: 0.25em;
}
div.tooltipster-categorized-elements label {
    margin-bottom: 1em;
    display: block;
}

#content .iconified-listing td.iconified-print a,
#content .iconified-listing td.iconified-signed a,
#content .iconified-listing td.iconified-confidential a,
#content .iconified-listing td.iconified-publishable a,
div.tooltipster-categorized-elements span.iconified-print,
div.tooltipster-categorized-elements span.iconified-confidential,
div.tooltipster-categorized-elements span.iconified-publishable,
div.tooltipster-categorized-elements span.iconified-signed {
    font-family: 'Font Awesome 5 Free';
    font-size: 110%;
    cursor: help;
    font-weight: bold;
}

/* need to use !important to have a border-bottom because it is removed using !important in base.css... */
#content .iconified-listing td.iconified-print a.editable,
#content .iconified-listing td.iconified-signed a.editable,
#content .iconified-listing td.iconified-confidential a.editable,
#content .iconified-listing td.iconified-publishable a.editable {
    border-bottom: 1px dotted !important;
    padding-bottom: 2px;
    cursor: pointer;
}

#content .iconified-listing td.iconified-print a,
#content .iconified-listing td.iconified-signed a,
#content .iconified-listing td.iconified-confidential a.active,
#content .iconified-listing td.iconified-publishable a,
div.tooltipster-categorized-elements span.iconified-print,
div.tooltipster-categorized-elements span.iconified-confidential.active,
div.tooltipster-categorized-elements span.iconified-publishable,
div.tooltipster-categorized-elements span.iconified-signed {
    color: #EA0000;
}

#content .iconified-listing td.iconified-print a.active,
#content .iconified-listing td.iconified-signed a.active,
#content .iconified-listing td.iconified-confidential a,
#content .iconified-listing td.iconified-publishable a.active,
div.tooltipster-categorized-elements span.iconified-print.active,
div.tooltipster-categorized-elements span.iconified-signed.active,
div.tooltipster-categorized-elements span.iconified-confidential,
div.tooltipster-categorized-elements span.iconified-publishable.active {
    color: #75ad0a;
}

#content .iconified-listing td.iconified-print a.active:hover,
#content .iconified-listing td.iconified-signed a.active:hover,
#content .iconified-listing td.iconified-confidential a:hover,
#content .iconified-listing td.iconified-publishable a.active:hover {
    color: #0B8400!important;
}

#content .iconified-listing td.iconified-print a:hover,
#content .iconified-listing td.iconified-signed a:hover,
#content .iconified-listing td.iconified-confidential a.active:hover,
#content .iconified-listing td.iconified-publishable a:hover {
    color: #B50000!important;
}

#content .iconified-listing td.iconified-print a.deactivated,
#content .iconified-listing td.iconified-signed a.deactivated,
#content .iconified-listing td.iconified-confidential a.deactivated,
#content .iconified-listing td.iconified-publishable a.deactivated,
div.tooltipster-categorized-elements span.iconified-print.deactivated,
div.tooltipster-categorized-elements span.iconified-signed.deactivated,
div.tooltipster-categorized-elements span.iconified-confidential.deactivated,
div.tooltipster-categorized-elements span.iconified-publishable.deactivated {
    color: #999;
}

#content .iconified-listing td.iconified-print a.deactivated:hover,
#content .iconified-listing td.iconified-signed a.deactivated:hover,
#content .iconified-listing td.iconified-confidential a.deactivated:hover,
#content .iconified-listing td.iconified-publishable a.deactivated:hover {
    color: #666!important;
}

#content .iconified-listing td.iconified-print a.error,
#content .iconified-listing td.iconified-signed a.error,
#content .iconified-listing td.iconified-confidential a.error,
#content .iconified-listing td.iconified-publishable a.error {
    color: #E58200;
}

#content .iconified-listing td.iconified-print a.error:hover,
#content .iconified-listing td.iconified-confidential a.error:hover,
#content .iconified-listing td.iconified-publishable a.error:hover {
    color: #C97200!important;
}

#content .iconified-listing td.iconified-print a:before,
div.tooltipster-categorized-elements span.iconified-print:before {
    content: "\f02f";
}

#content .iconified-listing td.iconified-publishable a:before,
div.tooltipster-categorized-elements span.iconified-publishable:before {
    content: "\f093";
}

#content .iconified-listing td.iconified-confidential a:before,
div.tooltipster-categorized-elements span.iconified-confidential:before {
    content: "\f3c1";
}

#content .iconified-listing td.iconified-confidential a.active:before,
div.tooltipster-categorized-elements span.iconified-confidential.active:before {
    content: "\f023";
}

#content .iconified-listing td.iconified-signed a:before,
div.tooltipster-categorized-elements span.iconified-signed:before {
    content: "\f044";
}

div.tooltipster-categorized-elements div.categorized_elements_details {
    display: inline-block;
    text-align: right;
    float: right;
}
div.tooltipster-categorized-elements div.categorized_elements_details span {
    float: left;
    width: 20px;
    text-align: center;
}
/* reduce select2 font-size */
.select2-container, .select2-results {
    font-size: 90%;
}
ul.select2-results {
    max-height: 400px;
}

/* add left margin to subcategories displayed in the select2 dropdown */
ul.select2-results span.subcategory {
    margin-left: 1em;
}
/* reduce space under selectable elements displayed in the dropdown */
ul.select2-results .select2-result-label {
    padding: 0px;
}

span.help {
    cursor: help;
}

#content div.categorized-elements a.manage-categorized-elements {
    vertical-align: middle;
}
